//=========================================================================//
//-------------------------------------------------------------------------//
// $Id: readtemp.cpp,v 1.6 2001/12/12 06:28:06 lanconel Exp $                                                                    //
//-------------------------------------------------------------------------//
//  Copyright (C) 1998-2001  Claudio Lanconelli                            //
//                                                                         //
//  e-mail: lancos@libero.it                                               //
//  http://www.LancOS.com                                                  //
//-------------------------------------------------------------------------//
//                                                                         //
// This program is free software; you can redistribute it and/or           //
// modify it under the terms of the GNU  General Public License            //
// as published by the Free Software Foundation; either version2 of        //
// the License, or (at your option) any later version.                     //
//                                                                         //
// This program is distributed in the hope that it will be useful,         //
// but WITHOUT ANY WARRANTY; without even the implied warranty of          //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       //
// General Public License for more details.                                //
//                                                                         //
// You should have received a copy of the GNU  General Public License      //
// along with this program (see COPYING);     if not, write to the         //
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. //
//                                                                         //
//-------------------------------------------------------------------------//
//=========================================================================//

#include <stdio.h>
#include <string.h>

#include "ds1621.h"
#include "readtemp.h"
#include "errcode.h"
#include "wait.h"

char *ReadTemperature(FormatType format, UnitType unit, int digits, int first, int last, int port)
{
	static char out_buffer[1024];
	int out_index = 0;
	DS1621 thermo;
	Wait w;

	if ( first > last ||
			first < 1 || first > 8 ||
			last  < 1 || last  > 8 )
	{
		return NULL;
	}

	thermo.Open(port);
	int j;
	for (j = first-1; j < last; j++)
	{
		thermo.SetAddress(j+1);
		thermo.Init();
		thermo.StartConversion();
	}
	w.WaitMsec(1000);
	for (j = first-1; j < last; j++)
	{
		thermo.SetAddress(j+1);
		thermo.WaitConversion();
		short val;
		int rv;
		if ( (rv = thermo.ReadTemperature(val)) )
		{
			sprintf(out_buffer+out_index, "Er%3d ", rv);
		}
		else
		{
			double temp = (unit == uCELSIUS) ?
								(double)val / 256.0 :
								(double)val * (9.0 / (256.0 * 5.0)) + 32.0;
			switch (digits)
			{
			case 0:
				sprintf(out_buffer+out_index, "%d%c%c ", (int)temp, format == fHTML ? '' : ' ', (unit == uCELSIUS) ? 'C' : 'F');
				break;
			case 2:
				sprintf(out_buffer+out_index, "%5.2f%c%c ", temp, format == fHTML ? '' : ' ', (unit == uCELSIUS) ? 'C' : 'F');
				break;
			case 3:
				sprintf(out_buffer+out_index, "%6.3f%c%c ", temp, format == fHTML ? '' : ' ', (unit == uCELSIUS) ? 'C' : 'F');
				break;
			case 4:
				sprintf(out_buffer+out_index, "%7.4f%c%c ", temp, format == fHTML ? '' : ' ', (unit == uCELSIUS) ? 'C' : 'F');
				break;
			default:
				sprintf(out_buffer+out_index, "%4.1f%c%c ", temp, format == fHTML ? '' : ' ', (unit == uCELSIUS) ? 'C' : 'F');
				break;
			}
		}
		out_index = strlen(out_buffer);
	}
	thermo.Close();

	return out_buffer;
}

int ReadTemperature(char status[N_THERM], short values[N_THERM], int first, int last, int port)
{
	DS1621 thermo;
	Wait w;

	if ( first > last ||
			first < 1 || first > 8 ||
			last  < 1 || last  > 8 )
	{
		return BADPARAM;
	}

	thermo.Open(port);
	int j;
	for (j = first-1; j < last; j++)
	{
		thermo.SetAddress(j+1);
		thermo.Init();
		thermo.StartConversion();
	}
	w.WaitMsec(1000);
	for (j = first-1; j < last; j++)
	{
		thermo.SetAddress(j+1);
		thermo.WaitConversion();
		status[j] = thermo.ReadTemperature(values[j]);
	//	thermo.StopConversion();
	}
	thermo.Close();

	return OK;
}

int ReadCounter(char status[N_THERM], short values[N_THERM], int first, int last, int port)
{
	DS1621 thermo;
	UBYTE val;
	Wait w;

	if ( first > last ||
			first < 1 || first > 8 ||
			last  < 1 || last  > 8 )
	{
		return BADPARAM;
	}

	thermo.Open(port);
	int j;
	for (j = first-1; j < last; j++)
	{
		thermo.SetAddress(j+1);
		thermo.Init();
		thermo.StartConversion();
	}
	w.WaitMsec(2000);
	for (j = first-1; j < last; j++)
	{
		thermo.SetAddress(j+1);
		status[j] = thermo.ReadCounter(val);
		values[j] = val;
	//	thermo.StopConversion();
	}
	thermo.Close();

	return OK;
}

int ReadSlope(char status[N_THERM], short values[N_THERM], int first, int last, int port)
{
	DS1621 thermo;
	UBYTE val;
	Wait w;

	if ( first > last ||
			first < 1 || first > 8 ||
			last  < 1 || last  > 8 )
	{
		return BADPARAM;
	}

	thermo.Open(port);
	int j;
	for (j = first-1; j < last; j++)
	{
		thermo.SetAddress(j+1);
		thermo.Init();
		thermo.StartConversion();
	}
	w.WaitMsec(2000);
	for (j = first-1; j < last; j++)
	{
		thermo.SetAddress(j+1);
		status[j] = thermo.ReadSlope(val);
		values[j] = val;
	//	thermo.StopConversion();
	}
	thermo.Close();

	return OK;
}
